use std::ops::Deref;

use telers::{
    errors::SessionErrorKind,
    methods::GetChatMember,
    types::User,
    types::{ChatMember, Message},
    Bot,
};

#[derive(Clone, Default, Debug)]
pub enum TargetUser {
    Id(i64),
    Reply(Message),
    CurrentMessage(Message),
    #[default]
    None,
}

impl From<Option<i64>> for TargetUser {
    fn from(value: Option<i64>) -> Self {
        match value {
            Some(id) => TargetUser::Id(id),
            None => TargetUser::None,
        }
    }
}

impl TargetUser {
    #[allow(dead_code)]
    pub fn exist(&self) -> bool {
        match self {
            Self::Id(_id) => true,
            Self::Reply(msg) => {
                if let Some(replied_msg) = msg.reply_to_message() {
                    return replied_msg.from().map(|user| user.id).is_some();
                }
                false
            }
            Self::CurrentMessage(msg) => msg.from().map(|user| user.id).is_some(),
            Self::None => false,
        }
    }

    pub fn get_id(&self) -> Option<i64> {
        match self {
            Self::Id(id) => Some(*id),
            Self::Reply(msg) => {
                if let Some(replied_msg) = msg.reply_to_message() {
                    return replied_msg.from().map(|user| user.id);
                }
                None
            }
            Self::CurrentMessage(msg) => msg.from().map(|user| user.id),
            Self::None => None,
        }
    }

    fn _inner_get_user_name(&self, user: User) -> Option<Box<str>> {
        if let Some(user_name) = user.username.as_ref() {
            return Some(user_name.clone());
        } else {
            return Some(Box::from(user.full_name()));
        }
    }

    pub async fn get_user_name(&self, bot: &Bot, msg: &Message) -> Option<Box<str>> {
        match self {
            Self::Id(id) => {
                let get_chat_member_result: Result<ChatMember, SessionErrorKind> =
                    bot.send(GetChatMember::new(msg.chat().id(), *id)).await;

                if let Ok(member_kind) = get_chat_member_result {
                    match member_kind {
                        ChatMember::Owner(member) => self._inner_get_user_name(member.user),
                        ChatMember::Administrator(member) => self._inner_get_user_name(member.user),
                        ChatMember::Member(member) => self._inner_get_user_name(member.user),
                        ChatMember::Restricted(member) => self._inner_get_user_name(member.user),
                        ChatMember::Left(member) => self._inner_get_user_name(member.user),
                        ChatMember::Banned(member) => self._inner_get_user_name(member.user),
                    };
                }
                None
            }
            Self::Reply(msg) => {
                if let Some(replied_msg) = msg.reply_to_message() {
                    return replied_msg.from().unwrap().username.clone();
                }
                None
            }
            Self::CurrentMessage(msg) => {
                if let Some(replied_msg) = msg.from() {
                    return replied_msg.username.clone();
                }
                None
            }
            Self::None => None,
        }
    }
}
