use std::{collections::VecDeque, ops::Deref};

use telers::types::Message;

use crate::types::enums::{target_user::TargetUser, time_metrics::TimeDuration};

#[derive(Debug)]
pub enum Argument {
    User(TargetUser),
    Time(TimeDuration),
    Reason(String),
}

pub fn parse_args(
    args: &[Box<str>],
    message: &Message,
    command_type: &str,
) -> Option<Vec<Argument>> {
    if args.is_empty() && message.reply_to_message().is_none() {
        return None;
    }

    let mut raw_args: VecDeque<&str> = args.iter().map(|arg| arg.deref()).collect();
    let mut parsed_args: Vec<Argument> = Vec::new();

    let user_id = match message.reply_to_message() {
        Some(reply_message) => TargetUser::from(reply_message.from_id()),
        None => match raw_args.pop_front() {
            Some(arg) => TargetUser::from(arg.parse::<i64>().ok()),
            None => TargetUser::default(),
        },
    };

    parsed_args.push(Argument::User(user_id));

    match command_type {
        "mute" | "tban" => 'raw_time_parsing: {
            let time = match raw_args.pop_front() {
                Some(raw_time) => raw_time.parse::<i64>().unwrap_or(0),
                None => break 'raw_time_parsing,
            };

            if time == 0 {
                break 'raw_time_parsing;
            }

            if let Some(raw_time_metric) = raw_args.pop_front() {
                let time_metric = match TimeDuration::from(raw_time_metric, time) {
                    Some(metric) => metric,
                    None => TimeDuration::Days(time),
                };

                parsed_args.push(Argument::Time(time_metric));
            };
        }
        _ => {}
    }

    if !raw_args.is_empty() {
        let reason = raw_args.iter().fold(String::new(), |acc, v| acc + v + " ");

        parsed_args.push(Argument::Reason(reason.trim_end().to_string()));
    }

    Some(parsed_args)
}
